from m.cli import Meta

meta = Meta(
    help='continuous integration utilities',
    description="""
        The following commands provide local support in the continuous
        integration flow.
    """,
)
